<?php

declare(strict_types=1);

namespace Horde\Composer;

use Composer\Plugin\Capability\CommandProvider as CommandProviderCapability;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Output\OutputInterface;
use Composer\Command\BaseCommand;

class HordeReconfigureCommand extends BaseCommand
{
    protected function configure(): void
    {
        $this->setName('horde-reconfigure');
        $this->setDescription('Rewrite autogenerated configuration');
        $this->setHelp(
            <<<EOT
Horde Installer writes various symlinks and files with installation specific paths.
If you move your installation around or manually tinker with local files, you may need to re-run this.
EOT
        );
    }

    protected function execute(InputInterface $input, OutputInterface $output): int
    {
        if (method_exists($this, 'requireComposer')) {
            $composer = $this->requireComposer();
        } else {
            $composer = $this->getComposer();
        }
        if (!$composer) {
            die('Error: Command was run without a relation to composer itself');
        }
        $flow = new HordeReconfigureFlow(new IOAdapter\SymphonyOutputAdapter($output), $composer);
        return $flow->run();
    }
}
